import sqlite3
import logging

LOGGER = logging.Logger(__name__)
LOGGER.setLevel(logging.CRITICAL)


class Database:
    def __init__(self, database_file: str = 'database.db') -> None:
        self.database_file = database_file
        self.conn = None
        self.cursor = None

    def __enter__(self):
        self.connect()
        return self

    def __exit__(self, exc_type, exc_value, traceback):
        self.close_connection()

    def connect(self):
        try:
            self.conn = sqlite3.connect(self.database_file)
            self.cursor = self.conn.cursor()
            LOGGER.info("Connected to database")
        except sqlite3.Error as e:
            LOGGER.critical(f'Connection failed with error: {e}')

    def execute_query(self, query, params=None):
        try:
            if params:
                self.cursor.execute(query, params)
            else:
                self.cursor.execute(query)
            return self.cursor.fetchall()
        except sqlite3.Error as error:
            LOGGER.error("Error executing the query:", error)
            return None

    def commit_changes(self):
        try:
            self.conn.commit()
        except sqlite3.Error as error:
            LOGGER.error("Error committing the changes:", error)

    def close_connection(self):
        try:
            if self.cursor is not None:
                self.cursor.close()
            if self.conn is not None:
                self.conn.close()
            LOGGER.info("SQLite connection is closed.")
        except sqlite3.Error as error:
            LOGGER.error("Error closing the SQLite connection:", error)

    def close_connection(self):
        try:
            if self.cursor is not None:
                self.cursor.close()
            if self.conn is not None:
                self.conn.close()
            LOGGER.info("SQLite connection is closed.")
        except sqlite3.Error as error:
            LOGGER.error("Error closing the SQLite connection:", error)

    def create_database(self):
        try:
            self.connect()
            LOGGER.info(
                f"Database '{self.database_file}' created successfully.")
        except sqlite3.Error as e:
            LOGGER.error("Error creating the database:", e)

    def create_table(self, table_name, columns) -> bool:
        try:
            self.connect()
            query = f"CREATE TABLE IF NOT EXISTS {table_name} ({', '.join(columns)})"
            self.cursor.execute(query)
            self.conn.commit()
            LOGGER.info(f"Table '{table_name}' created successfully.")
            return True
        except sqlite3.Error as e:
            LOGGER.error("Error creating the table:", e)
            return False

    def delete_table(self, table_name) -> bool:
        try:
            self.connect()
            query = f"DROP TABLE {table_name}"
            self.cursor.execute(query)
            self.conn.commit()
            LOGGER.info(f"Table [{table_name}] dropped")
            return True
        except sqlite3.Error as e:
            LOGGER.error("Error dropping the table:", e)
            return False

    def insert_row(self, table_name, data) -> None:
        try:
            self.connect()
            query = f"INSERT INTO {table_name} VALUES ({','.join(['?'] * len(data))})"
            self.cursor.execute(query, data)
            self.conn.commit()
            LOGGER.info(f"Row inserted into table '{table_name}'")
        except sqlite3.Error as e:
            LOGGER.error(f"Error inserting row into table '{table_name}': {e}")

    def delete_row(self, table_name, where_clause) -> None:
        try:
            self.connect()
            query = f"DELETE FROM {table_name} WHERE {where_clause}"
            self.cursor.execute(query)
            self.conn.commit()
            LOGGER.info(f"Row deleted from table '{table_name}'")
        except sqlite3.Error as e:
            LOGGER.error(f"Error deleting row from table '{table_name}': {e}")

    def read_all_rows(self, table_name) -> list | None:
        try:
            self.connect()
            query = f"SELECT * FROM {table_name}"
            self.cursor.execute(query)
            rows = self.cursor.fetchall()
            LOGGER.info(f"Read {len(rows)} rows from table '{table_name}'")
            return rows
        except sqlite3.Error as e:
            LOGGER.error(f"Error reading rows from table '{table_name}': {e}")
            return None

    def get_all_tables(self) -> list:
        try:
            self.connect()
            query = "SELECT name FROM sqlite_master WHERE type='table';"
            self.cursor.execute(query)
            rows = self.cursor.fetchall()
            LOGGER.info(f'Read {len(rows)} tables from database')
            return rows
        except sqlite3.Error as e:
            LOGGER.error(f"Error reading from sqlite_master", e)
            return []



import calendar
import os
import typer
from datetime import datetime
from enum import Enum
from rich.table import Table
from .local_db import LocalDatabase


class ClockStatus(Enum):
    NONE = 0
    IN = 1
    OUT = 2


def create_directories(config_dir: str, data_dir: str):
    """Create the necessary directories if they don't exist."""
    if not os.path.exists(config_dir):
        os.makedirs(config_dir)
    if not os.path.exists(data_dir):
        os.makedirs(data_dir)


def create_file(filename: str):
    if not os.path.exists(filename):
        try:
            with open(filename, 'w+'):
                pass
        except:
            print("Failed to create clock file")
            return
        print(f"Created {filename}")
    else:
        print(f"{filename} already exists.")


def validate_month(month: str) -> int:
    if month.lower() == 'current':
        return datetime.now().month
    try:
        month_num = int(month)
        if 1 <= month_num <= 12:
            return month_num
        else:
            print(f"Invalid month number: {month}")
            raise typer.Exit(code=1)
    except ValueError:
        month_num = list(calendar.month_name).index(month.capitalize())
        if month_num > 0:
            return month_num
        else:
            print(f"Invalid month name: {month}")
            raise typer.Exit(code=1)


def add_entry(customer: str, action: str, config_dir: str, table_name: str):
    if customer is None:
        customer = typer.prompt("Customer")
    with LocalDatabase.Database(database_file=f"{config_dir}/database.db") as db:
        db.insert_row(table_name, (
            str(datetime.now().strftime("%Y-%m-%d")),
            str(datetime.now().strftime('%H:%M')),
            action,
            customer
        ))


def get_rows(config_dir: str, table_name: str, print_line_num: bool = False):
    with LocalDatabase.Database(database_file=f"{config_dir}/database.db") as db:
        table = Table()
        if print_line_num:
            table.add_column('')
        table.add_column('Date')
        table.add_column('Time')
        table.add_column('Action')
        table.add_column('Customer')
        for i, row in enumerate(db.read_all_rows(table_name), start=1):
            id_, timestamp, action, customer = row
            match action:
                case 'in':
                    action = '[green]in[/green]'
                case 'out':
                    action = '[red]out[/red]'
            if print_line_num:
                table.add_row(str(i), str(id_), timestamp, action, customer)
            else:
                table.add_row(str(id_), timestamp, action, customer)
        return (table)


def find_status_by_date(date: str, config_dir: str, table_name: str) -> ClockStatus:
    with LocalDatabase.Database(database_file=f"{config_dir}/database.db") as db:
        entries = db.read_all_rows(table_name)

    status = ClockStatus.NONE

    for row in entries:
        if row[0] == date:
            if row[2] == 'out':
                status = ClockStatus.OUT
            elif row[2] == 'in' and status != ClockStatus.OUT:
                status = ClockStatus.IN
    return status



import os
import pathlib
import tempfile
import typer
import typer.completion
from datetime import datetime
from rich import print
from rich.table import Table
from typing import Annotated
from .local_db import LocalDatabase
from .utils import (add_entry, ClockStatus, create_directories, create_file,
                    find_status_by_date, get_rows, validate_month)


CONFIG_DIR = pathlib.Path.home() / ".config/clockz"
DATA_DIR = CONFIG_DIR / "data"
CSV_FILE = f"{datetime.now().strftime('%B')}.csv"
CSV_FILE_PATH = DATA_DIR / CSV_FILE
DEFAULT_TABLE_NAME = f'data_{datetime.now().strftime("%Y")}_{datetime.now().strftime("%m")}'


app = typer.Typer(name="cxz")
config_app = typer.Typer(name="config", help="Configuration reletad commands.")
app.add_typer(config_app)


def main():
    create_directories(CONFIG_DIR, DATA_DIR)
    if not os.path.exists(CSV_FILE_PATH):
        create_file(CSV_FILE_PATH)
    create_db()
    create_db_table()
    app()


# TODO: Support time and date input / picker
@app.command(name='in')
def clock_in(customer: str = typer.Argument(None)):
    """Clock in for the day."""
    add_entry(customer, 'in', CONFIG_DIR, DEFAULT_TABLE_NAME)


# TODO: Support time and date input / picker
@app.command(name='out')
def clock_out(customer: str = typer.Argument(None)):
    """Clock out for the day."""
    add_entry(customer, 'out', CONFIG_DIR, DEFAULT_TABLE_NAME)


@app.command(name="show")
def clock_show(
    month: str = typer.Option(str(datetime.now().strftime('%m'))),
    year: str = typer.Option(str(datetime.now().strftime('%Y')))
):
    """Display clock-in/clock-out records."""
    month = validate_month(month)
    print(get_rows(CONFIG_DIR, DEFAULT_TABLE_NAME))


@config_app.command('dir')
def config_dir_command():
    """
    Print the DATA directory path.
    """
    print(DATA_DIR)


@config_app.command("show-tables")
def show_tables():
    """
    List all tables in the database.
    """
    with LocalDatabase.Database(database_file=f"{CONFIG_DIR}/database.db") as db:
        table = Table()
        table.add_column('Table')
        for row in db.get_all_tables():
            table.add_row(row[0])
        print(table)


@config_app.command("create-db")
def create_db():
    """Create the local database file."""
    with LocalDatabase.Database(database_file=f"{CONFIG_DIR}/database.db") as db:
        db.create_database()


@config_app.command('create-table')
def create_db_table(
    month: Annotated[str, typer.Option(..., prompt=True)] = str(
        datetime.now().strftime("%m")),
    year: Annotated[str, typer.Option(..., prompt=True)] = str(
        datetime.now().strftime("%Y"))
):
    """Create a new database table."""
    with LocalDatabase.Database(database_file=f"{CONFIG_DIR}/database.db") as db:
        if not db.create_table(
            table_name=f"data_{year}_{month}",
            columns=["date TEXT", "time TEXT", "action TEXT", "customer TEXT"]
        ):
            print('[red]Database table failed to create[/red]')


@config_app.command()
def drop_table(month: str, year: str):
    """Erase all months' records."""
    table_name = f"data_{year}_{month}"
    with LocalDatabase.Database(database_file=f"{CONFIG_DIR}/database.db") as db:
        typer.confirm(
            f"You sure you want to delete all entries for the month {month}.{year}?", abort=True)
        if db.delete_table(table_name):
            print(f'[green]Table {table_name} dropped[/green]')
        else:
            print(f'[red]Could not drop table [{table_name}][/red]')


@app.command()
def delete():
    """Delete a specific clock-in/clock-out record."""
    print(get_rows(CONFIG_DIR, DEFAULT_TABLE_NAME, True))

    entries = []
    with LocalDatabase.Database(database_file=f"{CONFIG_DIR}/database.db") as db:
        reader = db.read_all_rows(DEFAULT_TABLE_NAME)
        for i, row in enumerate(reader, start=1):
            entries.append((i,)+row)

        line_number = typer.prompt(
            '\nEnter the line number of the entry you want to delete', type=int)

        for entry in entries:
            if entry[0] == line_number:
                _, date, time, action, customer = entry

        typer.confirm("Are you sure you want to delete?", abort=True)

        db.delete_row(DEFAULT_TABLE_NAME,
                      f"date = '{date}' AND time = '{time}' AND action = '{action}' AND customer = '{customer}'")


@app.command()
def status():
    """Display the clock-in/out status for today."""
    date = datetime.now().strftime("%Y-%m-%d")
    match find_status_by_date(date, CONFIG_DIR, DEFAULT_TABLE_NAME):
        case ClockStatus.NONE:
            print("No clocking entry found for today")
        case ClockStatus.IN:
            print("Found a clock-in entry for today")
        case ClockStatus.OUT:
            print("found clock-out entry for today")


@app.command('edit')
def edit_table(
    month: Annotated[str, typer.Option(..., prompt=True)] = str(
        datetime.now().strftime("%m")),
    year: Annotated[str, typer.Option(..., prompt=True)] = str(
        datetime.now().strftime("%Y"))
):
    """Edit a database's table."""
    table_name = f"data_{year}_{month}"

    with LocalDatabase.Database(database_file=f"{CONFIG_DIR}/database.db") as db:
        with tempfile.NamedTemporaryFile(mode="w", delete=False) as temp_file:
            reader = db.read_all_rows(table_name)
            for row in reader:
                temp_file.write("\t".join(row) + "\n")

        editor = os.environ.get("EDITOR", "nano")
        os.system(f"{editor} {temp_file.name}")

        with open(temp_file.name, "r") as updated_file:
            updated_rows = [line.strip().split("\t")
                            for line in updated_file.readlines()]

        db.delete_table(table_name)
        db.create_table(
            table_name, ["date TEXT", "time TEXT", "action TEXT", "customer TEXT"])
        for row in updated_rows:
            db.insert_row(table_name, row)

        print(f"[green]Table {table_name} updated[/green]")


if __name__ == "__main__":
    app()



from setuptools import setup, find_packages

setup(
    name='cloxz',
    version='0.4.3',
    packages=find_packages(),
    install_requires=[
        'typer',
        'rich'
    ],
    entry_points={
        'console_scripts': [
            'cxz=clock.main:main',
        ],
    },
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown'
)



import click
from utils import add_clock_entry


@click.command()
@click.argument('text', required=False)
def clock_in(filename: str, text: str, action: str) -> None:
    if text in (None, ""):
        text = click.prompt('Enter text for the clock entry', type=str)
    add_clock_entry(filename, text, action)
